/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2024 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "fdcan.h"
#include "i2c.h"
#include "quadspi.h"
#include "rng.h"
#include "tim.h"
#include "usart.h"
#include "gpio.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */

/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */

/* This is the buffer used to store the received CAN message from the STG-800. */
uint8_t Buffer[8];

uint8_t TestBuffer[8] = {1,2,3,4,5,6,7,8};

/* This is the variable used to store the RxHeader data length code. */
int DLC;

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{

  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_FDCAN2_Init();
  MX_QUADSPI1_Init();
  MX_TIM6_Init();
  MX_USART2_UART_Init();
  MX_I2C2_Init();
  MX_USART3_UART_Init();
  MX_RNG_Init();
  MX_FDCAN1_Init();
  /* USER CODE BEGIN 2 */

  /* Functions used to configure the FDCAN filters. These functions can be found in fdcan.c
   * All functions can be highlighted and right clicked to open the declaration. */
  FDCAN1FilterConfig();
  FDCAN2FilterConfig();

  /* Function used to configure the FDCAN2 Transmission header. This function can be found in fdcan.c */
  FDCAN2TxHeaderConfig();

  /* Functions used to start the FDCAN modules */
  HAL_FDCAN_Start(&hfdcan1);
  HAL_FDCAN_Start(&hfdcan2);

  /* HAL functions that enable interrupts. */
  HAL_FDCAN_ActivateNotification(&hfdcan1, FDCAN_IT_RX_FIFO0_NEW_MESSAGE, 0);
  HAL_FDCAN_ActivateNotification(&hfdcan2, FDCAN_IT_RX_FIFO1_NEW_MESSAGE, 0);

  /* Functions used to pull both CAN_ON pins low, as default condition. */
  HAL_GPIO_WritePin(CAN1_ON_GPIO_Port, CAN1_ON_Pin, GPIO_PIN_SET);
  HAL_GPIO_WritePin(CAN2_ON_GPIO_Port, CAN2_ON_Pin, GPIO_PIN_SET);

  /* Delay of 1000ms so all initializations have run before continuing. */
  HAL_Delay(1000);

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {

	  /* This functions gets the CAN message at the beginning of the while loop
	   * for the purpose of getting the RxHeader of the received CAN message
	   * so we can check what size the DLC of the CAN message is.
	   * The variable 'Buffer' is also used to store the content of the CAN message.
	   * This function can be found at the end in fdcan.c */
	  FDCanChooseSize(Buffer);

	  /* Now that a CAN message has been received, the header can be checked for the DLC.
	   * That information is saved in the variable 'DLC'. */
	  DLC = RxHeader.DataLength;

	  /* This function uses the 'DLC' variable to choose the correct DLC for the CAN communication.
	   * This function is part of fdcan.c */
	  FDCAN1_Rx(DLC);


	  /* -------------------------------------------------------------------------------------------------------------- *
	   * These if functions check if the identifier in the Rx Header of the received CAN message is equal to either		*
	   * CAN, ONEWIRE, or RS232. These addresses are defined in main.h													*
	   * Depending on what CAN address is transmitted to the GT-900, the CAN message is transfered in the GT-900 to a 	*
	   * different BUS and the LED on the GT-900 behaves differently.													*
	   *																												*
	   * CAN 		= 0x610 	-> RED LED ON																			*
	   * RS232  	= 0x630 	-> GREEN LED ON																			*
	   * RS485		= 0x640 	-> GREEN LED FLASHING 																	*
	   * -------------------------------------------------------------------------------------------------------------- */

	  if(RxHeader.Identifier == CAN)
	  {


		  /* These two functions turn the green LED off and the red LED on permanently, as long as the conditions of the if
		   * function are met*/
		  HAL_GPIO_WritePin(LED_GN_GPIO_Port, LED_GN_Pin, GPIO_PIN_RESET);
		  HAL_GPIO_WritePin(LED_RT_GPIO_Port, LED_RT_Pin, GPIO_PIN_SET);


		  /* This function takes the CAN message, received in the function "FDCAN1_Rx" (TempBuffer), taking the DLC (DLC)
		   * choosing the appropriate buffer declared in fdcan.c, and transmits it to the second CANBUS on the GT-900
		   * This function is part of fdcan.c */
		  FDCAN2_Tx(DLC);
	  }

	  if(RxHeader.Identifier == RS232)
	  {
		  /* These two functions turn the red LED off and the green LED on permanently, as long as the conditions of the if
		   * function are met*/
		  HAL_GPIO_WritePin(LED_RT_GPIO_Port, LED_RT_Pin, GPIO_PIN_RESET);
		  HAL_GPIO_WritePin(LED_GN_GPIO_Port, LED_GN_Pin, GPIO_PIN_SET);

		  /* This function takes the data from the received CAN message 'Buffer' and the message length 'DLC'
		   * and transmits it via the RS232 BUS. This function can be found in usart.c */
		  RS232_Tx(Buffer, DLC);
	  }

	  if(RxHeader.Identifier == RS485)
	  {
		  /* These two functions turn the red LED off and the green LED on permanently, as long as the conditions of the if
		   * function are met*/
		  HAL_GPIO_WritePin(LED_RT_GPIO_Port, LED_RT_Pin, GPIO_PIN_RESET);
		  HAL_GPIO_TogglePin(LED_GN_GPIO_Port, LED_GN_Pin);

		  /* This function takes the data from the received CAN message 'Buffer' and the message length 'DLC'
		   * and transmits it via the RS232 BUS. This function can be found in usart.c */
		  RS485_Tx(Buffer, DLC);
	  }
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Configure the main internal regulator output voltage
  */
  HAL_PWREx_ControlVoltageScaling(PWR_REGULATOR_VOLTAGE_SCALE1_BOOST);

  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI48|RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.HSI48State = RCC_HSI48_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = RCC_PLLM_DIV1;
  RCC_OscInitStruct.PLL.PLLN = 20;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = RCC_PLLQ_DIV2;
  RCC_OscInitStruct.PLL.PLLR = RCC_PLLR_DIV2;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_4) != HAL_OK)
  {
    Error_Handler();
  }
}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
